#include <Sensor.hpp>
#include <Arduino.h>

template<typename T>
inline size_t writeType(byte *buffer, T val) {
	memcpy(buffer, &val, sizeof(T));

	return sizeof(T);
}

Sensor::Sensor(int id, float resolution, float accuracy, DallasTemperature &bus, DeviceAddress address) : id(id), resolution(resolution), accuracy(accuracy), bus(bus) {
	memcpy(this->address, address, sizeof(DeviceAddress));
};

float Sensor::read() const {
	if(!bus.isConnected(address)) return NAN;

	bus.requestTemperaturesByAddress(address);
	return bus.getTempC(address);
}

bool Sensor::makeDataPacket(byte (&buffer)[dataPacketSize]) const {
	float readVal = read();
	if(isnan(readVal)) return false;

	size_t written = 0;
	written += writeType(&buffer[written], id);
	written += writeType(&buffer[written], readVal);

	return true;
}

void Sensor::registerSensor(byte (&buffer)[serializedSize]) const {
	size_t written = 0;

	written += writeType(&buffer[written], id);
	written += writeType(&buffer[written], resolution);
	written += writeType(&buffer[written], accuracy);
}